-----------------------------------
-- Setting up scope and libs
-----------------------------------

local AddonName = "iGear"
iGear = LibStub("AceAddon-3.0"):NewAddon(AddonName, "AceEvent-3.0");

local QTip = LibStub("LibQTip-1.0");
local Crayon = LibStub("LibCrayon-3.0");

local L = LibStub("AceLocale-3.0"):GetLocale(AddonName);

-----------------------------------------
-- Variables, functions and colors
-----------------------------------------

local Tooltip;

local COLOR_GOLD = "|cfffed100%s|r";

local NULL = "";
local REPAIR_COST = 0;
local SHOW_REPAIR_COST = false;

-- general slots
local SLOT_NAME, SLOT_ID, SLOT_IS_EQUIPPED, SLOT_COLOR, SLOT_ITEM_NAME = 1, 2, 9, 10, 11;
-- durability
local SLOT_CAN_DURABILITY, SLOT_DURABILITY, SLOT_REPAIR_COST = 3, 4, 6;
-- gems
local SLOT_GEMS_EMPTY = 5;
-- enchants
local SLOT_CAN_ENCHANT, SLOT_ENCHANT = 7, 8;

iGear.EquipSlots = {
	{"HeadSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"NeckSlot", 0, false, 0, 0, 0, false, 0, false, "", ""},
	{"ShoulderSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"BackSlot", 0, false, 0, 0, 0, true, 0, false, "", ""},
	{"ChestSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"WristSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"HandsSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"WaistSlot", 0, true, 0, 0, 0, false, 0, false, "", ""},
	{"LegsSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"FeetSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"Finger0Slot", 0, false, 0, 0, 0, false, 0, false, "", ""},
	{"Finger1Slot", 0, false, 0, 0, 0, false, 0, false, "", ""},
	{"Trinket0Slot", 0, false, 0, 0, 0, false, 0, false, "", ""},
	{"Trinket1Slot", 0, false, 0, 0, 0, false, 0, false, "", ""},
	{"MainHandSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"SecondaryHandSlot", 0, true, 0, 0, 0, true, 0, false, "", ""},
	{"RangedSlot", 0, false, 0, 0, 0, false, 0, false, "", ""}
};
local EquipSlots = iGear.EquipSlots;

local function tclear(t, wipe)
	if( type(t) ~= "table" ) then return end;
	for k in pairs(t) do
		t[k] = nil;
	end
	t[''] = 1;
	t[''] = nil;
	if( wipe ) then
		t = nil;
	end
end

--------------------------------------------------------------------
-- Initialize static table changes according to players class
--------------------------------------------------------------------

-- TO BE REMOVED WITH TOC 50000 AND ABOVE!
--=========================================
-- There are some differents between the classes. Hunter bows (ranged slot), can be enchanted.
-- Paladin Relics (ranged slot, too), canNOT be enchanted. So, when I play a Hunter, ranged SLOT_CAN_ENCHANT must be true.
-- Let's do this job in a separate scope!
do
	local _, class = UnitClass("player");
	for _, slot in ipairs(EquipSlots) do
		
		-- WARRIOR can wear bows
		-- ROGUE can wear bows
		-- HUNTER can wear bows
		if( slot[SLOT_NAME] == "RangedSlot" ) then
			if( class == "WARRIOR" or class == "ROGUE" or class == "HUNTER" ) then
				slot[SLOT_CAN_ENCHANT] = true;
			end
		end
		
	end
end

-----------------------------
-- Setting up the feed
-----------------------------

local function Feed_OnClick(_, button)
	-- nothing to do at the moment...
end

local function Feed_OnEnter(anchor)
	-- when there are no problems, we don't need the tooltip.
	if( iGear:GetProblems() == 0 and REPAIR_COST == 0 ) then
		return;
	end

	-- force other i-Series qtip's to hide, when this tip is to be shown
	for k, v in QTip:IterateTooltips() do
		if( type(k) == "string" and strsub(k, 1, 6) == "iSuite" ) then
			v:Release(k);
		end
	end
	
	Tooltip = QTip:Acquire("iSuite"..AddonName);
	Tooltip:SetAutoHideDelay(0.1, anchor);
	Tooltip:SmartAnchorTo(anchor);
	iGear:UpdateTooltip();
	Tooltip:Show();
end

iGear.Feed = LibStub("LibDataBroker-1.1"):NewDataObject(AddonName, {
	type = "data source",
	text = "",
	icon = "Interface\\Minimap\\Tracking\\Repair",
	OnClick = Feed_OnClick,
	OnEnter = Feed_OnEnter,
});

----------------------
-- OnInitialize
----------------------

function iGear:OnInitialize()
	for _, slot in ipairs(EquipSlots) do
		slot[SLOT_ID] = GetInventorySlotInfo(slot[SLOT_NAME]);
	end
	
	self:RegisterEvent("PLAYER_DEAD", "DurabilityChange");
	self:RegisterEvent("PLAYER_UNGHOST", "DurabilityChange");
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "DurabilityChange");
	self:RegisterEvent("UPDATE_INVENTORY_DURABILITY", "DurabilityChange");
end

----------------------------
-- Create own tooltip
----------------------------

--/script for k, v in pairs(_G) do if( strsub(k, 1, 7) == "ScanTip") then print(k) end end

CreateFrame("GameTooltip", "iGearScanTip", nil, "GameTooltipTemplate");
iGearScanTip:SetOwner(WorldFrame, "ANCHOR_NONE");

local function ScanEmptySocketSlotsAndRepairCost(slotnum)
	local empty = 0;
	
	iGearScanTip:ClearLines();
	local equipped, _, cost = iGearScanTip:SetInventoryItem("player", slotnum);
	
	for i = 1, iGearScanTip:NumLines() do
		local text = _G["iGearScanTipTextLeft"..i];
		if( text ) then
			text = text:GetText();
		end
		
		if( string.find(strlower(L["SocketTexts"]), strlower(text..","), 1) ) then
			empty = empty + 1;
		end
	end
	
	return equipped, empty, cost;
end

--------------------------------------------
-- EventHandler and Problem functions
--------------------------------------------

function iGear:GetProblem(slot)
	if( slot[SLOT_IS_EQUIPPED] ) then
		--if( slot[SLOT_CAN_DURABILITY] and slot[SLOT_DURABILITY] ~= 100 ) then
	--		return true;
		if( slot[SLOT_GEMS_EMPTY] > 0 ) then
			return true;
		elseif( slot[SLOT_CAN_ENCHANT] and slot[SLOT_ENCHANT] == 0 ) then
			return true;
		end
	end
	
	return false;
end

function iGear:GetProblems()
	local problems = 0;
	
	for _, slot in ipairs(EquipSlots) do
		if( self:GetProblem(slot) ) then
			problems = problems + 1;
		end
	end
	
	return problems;
end

function iGear:DurabilityChange()

	REPAIR_COST = 0;
	SHOW_REPAIR_COST = false;
	
	local repair = 0;
	local smallest_percent = 100;

	for _, slot in ipairs(EquipSlots) do
		local equipped, emptyGems, cost = ScanEmptySocketSlotsAndRepairCost(slot[SLOT_ID]);
		
		if( not equipped ) then
			slot[SLOT_IS_EQUIPPED] = false;
		else
			slot[SLOT_IS_EQUIPPED] = true;
			slot[SLOT_CAN_DURABILITY] = true;
			
			-- check and calculate durability
			local durability = 0;
			if( slot[SLOT_CAN_DURABILITY] ) then			
				local broken = GetInventoryItemBroken("player", slot[SLOT_ID]);
				
				if( not broken ) then
					local current, maximum = GetInventoryItemDurability(slot[SLOT_ID]);
					
					if( not current or not maximum ) then
						slot[SLOT_CAN_DURABILITY] = false;
					else
						durability = 100 * current / maximum;
						
						if( durability - abs(durability) >= 0.5 ) then
							durability = ceil(durability);
						else
							durability = floor(durability);
						end
					end
				end
				
				if( slot[SLOT_CAN_DURABILITY] and durability < smallest_percent ) then
					smallest_percent = durability;
				end
			end
			slot[SLOT_DURABILITY] = durability;
			
			-- check for empty sockets and repaircost
			slot[SLOT_GEMS_EMPTY], slot[SLOT_REPAIR_COST] = emptyGems, cost;
			repair = repair + cost;
			
			-- retrieve item color, item name and enchant
			local _, _, color, enchant, name = string.find(
				GetInventoryItemLink("player", slot[SLOT_ID]),
				"|c%x%x(%x*)|Hitem:%d+:(%d+):%d+:%d+:%d+:%d+:%-?%d+:%-?%d+:%d+:%d+|h%[([^%]]*)%]"
			);
			slot[SLOT_COLOR] = color;
			slot[SLOT_ITEM_NAME] = name;
			
			-- check for enchants
			if( slot[SLOT_CAN_ENCHANT] ) then
				slot[SLOT_ENCHANT] = tonumber(enchant);
			end
			
		end
	end
	
	REPAIR_COST = repair;
	self.Feed.text = ("|cff%s%d%s|r"):format(Crayon:GetThresholdHexColor(smallest_percent), smallest_percent, "%");
	
	local problems = self:GetProblems();
	if( problems and problems ~= 0 ) then
		self.Feed.text = ("%s |cffff0000%d %s|r"):format(self.Feed.text, problems, problems == 1 and L["Alert"] or L["Alerts"]);
	end
	
end

-----------------------
-- UpdateTooltip
-----------------------

local function GetHintText(slot)
	local text = "";
	
	if( slot[SLOT_GEMS_EMPTY] ~= 0 ) then
		text = ("|cff%s%d %s|r"):format("ff0000", slot[SLOT_GEMS_EMPTY], slot[SLOT_GEMS_EMPTY] == 1 and L["Gem"] or L["Gems"]);
	end
	
	if( slot[SLOT_CAN_ENCHANT] and slot[SLOT_ENCHANT] == 0 ) then
		text = ("%s%s|cff00ffff%s|r"):format(text, text ~= "" and ", " or NULL, L["Enchant"]);
	end
	
	return text;
end

function iGear:UpdateTooltip()
	Tooltip:Clear();
	
	Tooltip:SetColumnLayout(4, "RIGHT", "LEFT", "LEFT");
	Tooltip:AddHeader("", L["Slot"], L["Item"], L["Hints"]);
	
	for _, slot in ipairs(EquipSlots) do
		if( (( slot[SLOT_CAN_DURABILITY] and slot[SLOT_DURABILITY] ~= 100 ) or self:GetProblem(slot) ) and slot[SLOT_IS_EQUIPPED] ) then
			Tooltip:AddLine(
				slot[SLOT_CAN_DURABILITY] and ("|cff%s%d%s|r"):format(Crayon:GetThresholdHexColor(slot[SLOT_DURABILITY]), slot[SLOT_DURABILITY], "%") or NULL,
				(COLOR_GOLD):format(L[slot[SLOT_NAME]]),
				("|cff%s%s|r"):format(slot[SLOT_COLOR], slot[SLOT_ITEM_NAME]),
				GetHintText(slot)
			);
		end
	end
	
	if( REPAIR_COST > 0 ) then
		Tooltip:AddLine(" ");
		Tooltip:AddLine("", (COLOR_GOLD):format(L["Repair Costs"]), GetMoneyString(REPAIR_COST));
	end
end